@@ -1,8 +1,4 @@
 #include "queue.h"
-/* Alloca is defined in stdlib.h in NetBSD/FreeBSD */
-#if !defined(__NetBSD__) && !defined(__FreeBSD__)
-#include <alloca.h>
-#endif
 #include <limits.h>
 #include <ctype.h>
 #include <dirent.h>
@@ -32,6 +28,8 @@
 #include "log.h"
 #include "logrotate.h"
 
+struct logInfoHead logs;
+
 #if !defined(GLOB_ABORTED) && defined(GLOB_ABEND)
 #define GLOB_ABORTED GLOB_ABEND
 #endif
@@ -284,7 +282,7 @@ static int readModeUidGid(const char *configFile, int lineNum, char *key,
                           gid_t *pGid)
 {
     char u[200], g[200];
-    unsigned int m;
+    unsigned int m = 0;
     char tmp;
     int rc;
 
@@ -395,6 +393,11 @@ static int mkpath(const char *path, mode_t mode, uid_t uid, gid_t gid) {
     int rv;
     char *copypath = strdup(path);
 
+    if (!copypath) {
+        message(MESS_ERROR, "can not allocate memory\n");
+        return 1;
+    }
+
     rv = 0;
     pp = copypath;
     while (rv == 0 && (sp = strchr(pp, '/')) != NULL) {
@@ -437,7 +440,7 @@ static int checkFile(const char *fname)
 /* Used by qsort to sort filelist */
 static int compar(const void *p, const void *q)
 {
-    return strcoll(*((char **) p), *((char **) q));
+    return strcoll(*((char * const*) p), *((char * const*) q));
 }
 
 /* Free memory blocks pointed to by pointers in a 2d array and the array itself */
@@ -449,11 +452,24 @@ static void free_2d_array(char **array, int lines_count)
     free(array);
 }
 
-static void copyLogInfo(struct logInfo *to, struct logInfo *from)
+#define MEMBER_COPY(dest, src) \
+    do { \
+        if (src && rv == 0) { \
+            (dest) = strdup(src); \
+            if ((dest) == NULL) { \
+                message(MESS_ERROR, "can not allocate memory\n"); \
+                rv = 1; \
+            } \
+        } else { \
+            (dest) = NULL; \
+        } \
+    } while (0)
+static int copyLogInfo(struct logInfo *to, const struct logInfo *from)
 {
+    int rv = 0;
+
     memset(to, 0, sizeof(*to));
-    if (from->oldDir)
-        to->oldDir = strdup(from->oldDir);
+    MEMBER_COPY(to->oldDir, from->oldDir);
     to->criterium = from->criterium;
     to->weekday = from->weekday;
     to->threshold = from->threshold;
@@ -463,26 +479,16 @@ static void copyLogInfo(struct logInfo *to, struct logInfo *from)
     to->rotateMinAge = from->rotateMinAge;
     to->rotateAge = from->rotateAge;
     to->logStart = from->logStart;
-    if (from->pre)
-        to->pre = strdup(from->pre);
-    if (from->post)
-        to->post = strdup(from->post);
-    if (from->first)
-        to->first = strdup(from->first);
-    if (from->last)
-        to->last = strdup(from->last);
-    if (from->preremove)
-        to->preremove = strdup(from->preremove);
-    if (from->logAddress)
-        to->logAddress = strdup(from->logAddress);
-    if (from->extension)
-        to->extension = strdup(from->extension);
-    if (from->compress_prog)
-        to->compress_prog = strdup(from->compress_prog);
-    if (from->uncompress_prog)
-        to->uncompress_prog = strdup(from->uncompress_prog);
-    if (from->compress_ext)
-        to->compress_ext = strdup(from->compress_ext);
+    MEMBER_COPY(to->pre, from->pre);
+    MEMBER_COPY(to->post, from->post);
+    MEMBER_COPY(to->first, from->first);
+    MEMBER_COPY(to->last, from->last);
+    MEMBER_COPY(to->preremove, from->preremove);
+    MEMBER_COPY(to->logAddress , from->logAddress);
+    MEMBER_COPY(to->extension, from->extension);
+    MEMBER_COPY(to->compress_prog, from->compress_prog);
+    MEMBER_COPY(to->uncompress_prog, from->uncompress_prog);
+    MEMBER_COPY(to->compress_ext, from->compress_ext);
     to->flags = from->flags;
     to->shred_cycles = from->shred_cycles;
     to->createMode = from->createMode;
@@ -493,13 +499,23 @@ static void copyLogInfo(struct logInfo *to, struct logInfo *from)
     to->olddirMode = from->olddirMode;
     to->olddirUid = from->olddirUid;
     to->olddirGid = from->olddirGid;
+
     if (from->compress_options_count) {
         poptDupArgv(from->compress_options_count, from->compress_options_list,
                     &to->compress_options_count,  &to->compress_options_list);
+        if (to->compress_options_list == NULL) {
+            message(MESS_ERROR, "can not allocate memory\n");
+            rv = 1;
+        }
     }
-    if (from->dateformat)
-        to->dateformat = strdup(from->dateformat);
+
+    MEMBER_COPY(to->dateformat, from->dateformat);
+
+    to->list = from->list;
+
+    return rv;
 }
+#undef MEMBER_COPY
 
 static void freeLogInfo(struct logInfo *log)
 {
@@ -520,14 +536,22 @@ static void freeLogInfo(struct logInfo *log)
     free(log->dateformat);
 }
 
-static struct logInfo *newLogInfo(struct logInfo *template)
+static struct logInfo *newLogInfo(const struct logInfo *template)
 {
     struct logInfo *new;
 
-    if ((new = malloc(sizeof(*new))) == NULL)
+    new = malloc(sizeof(*new));
+    if (new == NULL) {
+        message(MESS_ERROR, "can not allocate memory\n");
         return NULL;
+    }
+
+    if (copyLogInfo(new, template)) {
+        freeLogInfo(new);
+        free(new);
+        return NULL;
+    }
 
-    copyLogInfo(new, template);
     TAILQ_INSERT_TAIL(&logs, new, list);
     numLogs++;
 
@@ -541,6 +565,7 @@ static void removeLogInfo(struct logInfo *log)
 
     freeLogInfo(log);
     TAILQ_REMOVE(&logs, log, list);
+    free(log);
     numLogs--;
 }
 
@@ -559,7 +584,7 @@ static const char *crit_to_string(enum criterium crit)
         case ROT_HOURLY:    return "hourly";
         case ROT_DAYS:      return "daily";
         case ROT_WEEKLY:    return "weekly";
-        case ROT_MONTHLY:   return "montly";
+        case ROT_MONTHLY:   return "monthly";
         case ROT_YEARLY:    return "yearly";
         case ROT_SIZE:      return "size";
         default:            return "XXX";
@@ -629,9 +654,8 @@ static int readConfigPath(const char *path, struct logInfo *defConfig)
                     }
                 }
                 /* Alloc memory for file name */
-                if ((namelist[files_count] =
-                            (char *) malloc(strlen(dp->d_name) + 1))) {
-                    strcpy(namelist[files_count], dp->d_name);
+                namelist[files_count] = strdup(dp->d_name);
+                if (namelist[files_count] != NULL) {
                     files_count++;
                 } else {
                     free_2d_array(namelist, files_count);
@@ -662,11 +686,16 @@ static int readConfigPath(const char *path, struct logInfo *defConfig)
 
         for (i = 0; i < files_count; ++i) {
             assert(namelist[i] != NULL);
-            copyLogInfo(&defConfigBackup, defConfig);
+            if (copyLogInfo(&defConfigBackup, defConfig)) {
+                freeLogInfo(&defConfigBackup);
+                close(here);
+                free_2d_array(namelist, files_count);
+                return 1;
+            }
             if (readConfigFile(namelist[i], defConfig)) {
                 message(MESS_ERROR, "found error in file %s, skipping\n", namelist[i]);
                 freeLogInfo(defConfig);
-                copyLogInfo(defConfig, &defConfigBackup);
+                if (copyLogInfo(defConfig, &defConfigBackup)){} /* do not check, we are already in a error path */
                 freeLogInfo(&defConfigBackup);
                 result = 1;
                 continue;
@@ -680,10 +709,14 @@ static int readConfigPath(const char *path, struct logInfo *defConfig)
         close(here);
         free_2d_array(namelist, files_count);
     } else {
-        copyLogInfo(&defConfigBackup, defConfig);
+        if (copyLogInfo(&defConfigBackup, defConfig)) {
+            freeLogInfo(&defConfigBackup);
+            return 1;
+        }
+
         if (readConfigFile(path, defConfig)) {
             freeLogInfo(defConfig);
-            copyLogInfo(defConfig, &defConfigBackup);
+            if (copyLogInfo(defConfig, &defConfigBackup)){} /* do not check, we are already in a error path */
             result = 1;
         }
         freeLogInfo(&defConfigBackup);
@@ -736,6 +769,12 @@ int readAllConfigPaths(const char **paths)
     };
 
     tabooPatterns = malloc(sizeof(*tabooPatterns) * defTabooCount);
+    if (tabooPatterns == NULL) {
+        message(MESS_ERROR, "can not allocate memory\n");
+        return 1;
+    }
+
+
     for (i = 0; i < defTabooCount; i++) {
         int bytes;
         char *pattern = NULL;
@@ -790,7 +829,7 @@ static char* parseGlobString(const char *configFile, int lineNum,
             default:
                 if ('\n' == **ppos)
                     state = PGS_INIT;
-        };
+        }
 
         if (PGS_COMMENT == state)
             /* skip comment */
@@ -818,7 +857,7 @@ static char* parseGlobString(const char *configFile, int lineNum,
             globStringAlloc += GLOB_STR_REALLOC_STEP;
             ptr = realloc(globString, globStringAlloc);
             if (!ptr) {
-                /* out of memory */
+                message(MESS_ERROR, "can not realloc\n");
                 free(globString);
                 return NULL;
             }
@@ -839,9 +878,19 @@ static int globerr(const char *pathname, int theerr)
 {
     (void) pathname;
 
-    /* A missing directory is not an error, so return 0 */
-    if (theerr == ENOTDIR)
-        return 0;
+    /* prevent glob() from being aborted in certain cases */
+    switch (theerr) {
+        case ENOTDIR:
+            /* non-directory where directory was expected by the glob */
+            return 0;
+
+        case ENOENT:
+            /* most likely symlink with non-existent target */
+            return 0;
+
+        default:
+            break;
+    }
 
     glob_errno = theerr;
 
@@ -853,7 +902,7 @@ static int globerr(const char *pathname, int theerr)
     do { \
         free(newlog->what); \
         newlog->what = NULL; \
-    } while (0);
+    } while (0)
 #define RAISE_ERROR() \
     if (newlog != defConfig) { \
         state = STATE_ERROR; \
@@ -875,7 +924,6 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
     char **scriptDest = NULL;
     struct logInfo *newlog = defConfig;
     char *start, *chptr;
-    char *dirName;
     struct passwd *pw = NULL;
     int rc;
     struct stat sb, sb2;
@@ -899,10 +947,6 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
         .l_type = F_RDLCK
     };
 
-    /* FIXME: createOwner and createGroup probably shouldn't be fixed
-       length arrays -- of course, if we aren't run setuid it doesn't
-       matter much */
-
     fd = open(configFile, O_RDONLY);
     if (fd < 0) {
         message(MESS_ERROR, "failed to open config file %s: %s\n",
@@ -1027,7 +1071,6 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
 
     message(MESS_DEBUG, "reading config file %s\n", configFile);
 
-    start = buf;
     for (start = buf; start - buf < length; start++) {
         switch (state) {
             case STATE_DEFAULT:
@@ -1113,6 +1156,16 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
                                     "su\n", configFile, lineNum);
                             RAISE_ERROR();
                         }
+                        else if (newlog->suUid == NO_UID) {
+                            message(MESS_ERROR, "%s:%d no user for "
+                                    "su\n", configFile, lineNum);
+                            RAISE_ERROR();
+                        }
+                        else if (newlog->suGid == NO_GID) {
+                            message(MESS_ERROR, "%s:%d no group for "
+                                    "su\n", configFile, lineNum);
+                            RAISE_ERROR();
+                        }
 
                         newlog->flags |= LOG_FLAG_SU;
                     } else if (!strcmp(key, "create")) {
@@ -1362,8 +1415,13 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
 
                             /* accept only non-empty patterns to avoid exclusion of everything */
                             if (endtag < chptr) {
-                                tabooPatterns = realloc(tabooPatterns, sizeof(*tabooPatterns) *
+                                char **tmp = realloc(tabooPatterns, sizeof(*tabooPatterns) *
                                         (tabooCount + 1));
+                                if (tmp == NULL) {
+                                    message(MESS_ERROR, "can not allocate memory\n");
+                                    RAISE_ERROR();
+                                }
+                                tabooPatterns = tmp;
                                 bytes = asprintf(&pattern, "*%.*s", (int)(chptr - endtag), endtag);
 
                                 /* should test for malloc() failure */
@@ -1408,13 +1466,19 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
                         while (*endtag) {
                             int bytes;
                             char *pattern = NULL;
+                            char **tmp;
 
                             chptr = endtag;
                             while (!isspace((unsigned char)*chptr) && *chptr != ',' && *chptr)
                                 chptr++;
 
-                            tabooPatterns = realloc(tabooPatterns, sizeof(*tabooPatterns) *
+                            tmp = realloc(tabooPatterns, sizeof(*tabooPatterns) *
                                     (tabooCount + 1));
+                            if (tmp == NULL) {
+                                message(MESS_ERROR, "can not allocate memory\n");
+                                RAISE_ERROR();
+                            }
+                            tabooPatterns = tmp;
                             bytes = asprintf(&pattern, "%.*s", (int)(chptr - endtag), endtag);
 
                             /* should test for malloc() failure */
@@ -1495,12 +1559,21 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
                                 newlog->compress_prog);
 
                         compresscmd_base = strdup(basename(newlog->compress_prog));
+                        if (compresscmd_base == NULL) {
+                            message(MESS_ERROR, "can not allocate memory\n");
+                            RAISE_ERROR();
+                        }
                         /* we check whether we changed the compress_cmd. In case we use the appropriate extension
                            as listed in compress_cmd_list */
                         for(i = 0; i < compress_cmd_list_size; i++) {
                             if (!strcmp(compress_cmd_list[i].cmd, compresscmd_base)) {
                                 freeLogItem (compress_ext);
-                                newlog->compress_ext = strdup((char *)compress_cmd_list[i].ext);
+                                newlog->compress_ext = strdup(compress_cmd_list[i].ext);
+                                if (newlog->compress_ext == NULL) {
+                                    message(MESS_ERROR, "can not allocate memory\n");
+                                    free(compresscmd_base);
+                                    RAISE_ERROR();
+                                }
                                 message(MESS_DEBUG, "compress_ext was changed to %s\n", newlog->compress_ext);
                                 break;
                             }
@@ -1587,6 +1660,11 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
                     if (!newlog->compress_ext)
                         newlog->compress_ext = strdup(COMPRESS_EXT);
 
+                    if (!newlog->compress_prog || !newlog->uncompress_prog || !newlog->compress_ext) {
+                        message(MESS_ERROR, "can not allocate memory\n");
+                        goto error;
+                    }
+
                     /* Allocate a new logInfo structure and insert it into the logs
                        queue, copying the actual values from defConfig */
                     if ((newlog = newLogInfo(defConfig)) == NULL)
@@ -1615,6 +1693,8 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
                     newlog->files = NULL;
                     newlog->numFiles = 0;
                     for (argNum = 0; argNum < argc; argNum++) {
+                        char **tmp;
+
                         if (globerr_msg) {
                             free(globerr_msg);
                             globerr_msg = NULL;
@@ -1640,11 +1720,17 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
                             globResult.gl_pathc = 0;
                         }
 
-                        newlog->files =
-                            realloc(newlog->files,
+                        tmp = realloc(newlog->files,
                                     sizeof(*newlog->files) * (newlog->numFiles +
                                         globResult.
                                         gl_pathc));
+                        if (tmp == NULL) {
+                            message(MESS_ERROR, "can not allocate memory\n");
+                            logerror = 1;
+                            goto duperror;
+                        }
+
+                        newlog->files = tmp;
 
                         for (glob_count = 0; glob_count < globResult.gl_pathc; glob_count++) {
                             /* if we glob directories we can get false matches */
@@ -1670,6 +1756,11 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
 
                             newlog->files[newlog->numFiles] =
                                 strdup(globResult.gl_pathv[glob_count]);
+                            if (newlog->files[newlog->numFiles] == NULL) {
+                                message(MESS_ERROR, "can not allocate memory\n");
+                                logerror = 1;
+                                goto duperror;
+                            }
                             newlog->numFiles++;
                         }
 duperror:
@@ -1705,8 +1796,14 @@ duperror:
                         for (i = 0; i < newlog->numFiles; i++) {
                             char *ld;
                             char *dirpath;
+                            const char *dirName;
 
                             dirpath = strdup(newlog->files[i]);
+                            if (dirpath == NULL) {
+                                message(MESS_ERROR, "can not allocate memory\n");
+                                goto error;
+                            }
+
                             dirName = dirname(dirpath);
                             if (stat(dirName, &sb2)) {
                                 if (!(newlog->flags & LOG_FLAG_MISSINGOK)) {
@@ -1728,7 +1825,12 @@ duperror:
                                     continue;
                                 }
                             }
-                            ld = alloca(strlen(dirName) + strlen(newlog->oldDir) + 2);
+                            ld = malloc(strlen(dirName) + strlen(newlog->oldDir) + 2);
+                            if (ld == NULL) {
+                                message(MESS_ERROR, "can not allocate memory\n");
+                                free(dirpath);
+                                goto error;
+                            }
                             sprintf(ld, "%s/%s", dirName, newlog->oldDir);
                             free(dirpath);
 
@@ -1740,10 +1842,11 @@ duperror:
                             }
 
                             if (stat(dirName, &sb)) {
-                                if (errno == ENOENT && newlog->flags & LOG_FLAG_OLDDIRCREATE) {
+                                if (errno == ENOENT && (newlog->flags & LOG_FLAG_OLDDIRCREATE)) {
                                     int ret;
                                     if (newlog->flags & LOG_FLAG_SU) {
                                         if (switch_user(newlog->suUid, newlog->suGid) != 0) {
+                                            free(ld);
                                             goto error;
                                         }
                                     }
@@ -1751,10 +1854,12 @@ duperror:
                                             newlog->olddirUid, newlog->olddirGid);
                                     if (newlog->flags & LOG_FLAG_SU) {
                                         if (switch_user_back() != 0) {
+                                            free(ld);
                                             goto error;
                                         }
                                     }
                                     if (ret) {
+                                        free(ld);
                                         goto error;
                                     }
                                 }
@@ -1762,10 +1867,13 @@ duperror:
                                     message(MESS_ERROR, "%s:%d error verifying olddir "
                                             "path %s: %s\n", configFile, lineNum,
                                             dirName, strerror(errno));
+                                    free(ld);
                                     goto error;
                                 }
                             }
 
+                            free(ld);
+
                             if (sb.st_dev != sb2.st_dev
                                     && !(newlog->flags & (LOG_FLAG_COPYTRUNCATE | LOG_FLAG_COPY | LOG_FLAG_TMPFILENAME))) {
                                 message(MESS_ERROR,
@@ -1790,7 +1898,7 @@ duperror:
             case STATE_SKIP_LINE:
             case STATE_SKIP_LINE | STATE_SKIP_CONFIG:
                 if (*start == '\n')
-                    state = state & STATE_SKIP_CONFIG ? STATE_SKIP_CONFIG : STATE_DEFAULT;
+                    state = (state & STATE_SKIP_CONFIG) ? STATE_SKIP_CONFIG : STATE_DEFAULT;
                 break;
             case STATE_SKIP_LINE | STATE_LOAD_SCRIPT:
                 if (*start == '\n')
@@ -1807,10 +1915,10 @@ duperror:
                 if (*start != '\n') {
                     message(MESS_ERROR, "%s:%d, unexpected text after }\n",
                             configFile, lineNum);
-                    state = STATE_SKIP_LINE | (state & STATE_SKIP_CONFIG ? STATE_SKIP_CONFIG : 0);
+                    state = STATE_SKIP_LINE | ((state & STATE_SKIP_CONFIG) ? STATE_SKIP_CONFIG : 0);
                 }
                 else
-                    state = state & STATE_SKIP_CONFIG ? STATE_SKIP_CONFIG : STATE_DEFAULT;
+                    state = (state & STATE_SKIP_CONFIG) ? STATE_SKIP_CONFIG : STATE_DEFAULT;
                 break;
             case STATE_ERROR:
                 assert(newlog != defConfig);
@@ -1837,20 +1945,21 @@ duperror:
                         while (*endtag != '\n')
                             endtag--;
                         endtag++;
-                        *scriptDest = malloc(endtag - scriptStart + 1);
-                        strncpy(*scriptDest, scriptStart,
-                                endtag - scriptStart);
-                        (*scriptDest)[endtag - scriptStart] = '\0';
+                        *scriptDest = strndup(scriptStart, endtag - scriptStart);
+                        if (*scriptDest == NULL) {
+                            message(MESS_ERROR, "can not allocate memory\n");
+                            goto error;
+                        }
 
                         scriptDest = NULL;
                         scriptStart = NULL;
                     }
-                    state = state & STATE_SKIP_CONFIG ? STATE_SKIP_CONFIG : STATE_DEFAULT;
+                    state = (state & STATE_SKIP_CONFIG) ? STATE_SKIP_CONFIG : STATE_DEFAULT;
                 }
                 else {
                     state = (*start == '\n' ? 0 : STATE_SKIP_LINE) |
                         STATE_LOAD_SCRIPT |
-                        (state & STATE_SKIP_CONFIG ? STATE_SKIP_CONFIG : 0);
+                        ((state & STATE_SKIP_CONFIG) ? STATE_SKIP_CONFIG : 0);
                 }
                 break;
             case STATE_SKIP_CONFIG:
@@ -1895,7 +2004,7 @@ duperror:
                 }
                 break;
             default:
-                message(MESS_DEBUG,
+                message(MESS_FATAL,
                         "%s: %d: readConfigFile() unknown state\n",
                         configFile, lineNum);
         }
